-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}
local logTag = "bus"
M.type = "auxilliary"
M.relevantDevice = nil
M.defaultOrder = 1000

local htmlTexture = require("htmlTexture")
local destScreenName = nil
local destScreenCount = 1
local destScreenCountrear = 1
local destScreenTimer = 0
local eventDataNew = nil
local eventDataOld = nil
local destHtmlPath = nil
local displaytextarray = {}
local displaytextcolourarray = {}
local displaybgcolourarray = {}
local displayimgarray = {}
local displaytextreararray = {}
local displaytextcolourreararray = {}
local displaybgcolourreararray = {}
local displayinvertarray = {}
local displayinvertreararray = {}
local colourSign = false
local textColourdef = "#FFFFFF,#FFFFFF,#FFFFFF,#FFFFFF,#FFFFFF,#FFFFFF"
local bgColourdef = "#000000,#000000,#000000,#000000,#000000,#000000"
local invertdef = "0,0,0,0,0,0"

local function updateGFX(dt)
	if #(displaytextarray) > 1 or #(displaytextreararray) > 1 then
		destScreenTimer = destScreenTimer + dt
		if destScreenName and destScreenTimer >= tonumber(eventDataOld.interval) then
		  --front and side signs
		  eventDataNew.displaytext = displaytextarray[destScreenCount]
		  eventDataNew.displaytextcolour = displaytextcolourarray[destScreenCount]
		  eventDataNew.displaybgcolour = displaybgcolourarray[destScreenCount]
		  eventDataNew.displayimg = displayimgarray[destScreenCount]
		  eventDataNew.displayinvert = displayinvertarray[destScreenCount]
		  destScreenCount = destScreenCount + 1
		  if destScreenCount > #(displaytextarray) then destScreenCount = 1 end
		  --rear sign
		  eventDataNew.displaytextrear = displaytextreararray[destScreenCountrear]
		  eventDataNew.displaytextcolourrear = displaytextcolourreararray[destScreenCountrear]
		  eventDataNew.displaybgcolourrear = displaybgcolourreararray[destScreenCountrear]
		  eventDataNew.displayinvertrear = displayinvertreararray[destScreenCountrear]
		  eventDataNew.destScreenCountrear = destScreenCountrear
		  destScreenCountrear = destScreenCountrear + 1
		  if destScreenCountrear > #(displaytextreararray) then destScreenCountrear = 1 end
		  if destScreenCountrear == 1 and displaytextreararray[1] == "" then destScreenCountrear = destScreenCountrear + 1 end
		  htmlTexture.call(destScreenName, "update", eventDataNew)
		  destScreenTimer = destScreenTimer % tonumber(eventDataOld.interval)
		end
    end
end

local function split (inputstr, sep, def)
        if inputstr == nil then inputstr = def end
        if sep == nil then sep = "," end
        local t={}
        for str in string.gmatch(inputstr, "([^"..sep.."]+)") do
                table.insert(t, str)
        end
        return t
end

local function init(jbeamData)
	if eventDataOld then return end
  if jbeamData.destinationSignNode and v.data[jbeamData.destinationSignNode] then
    local destData = v.data[jbeamData.destinationSignNode]
    destScreenName = destData.destMaterialName or destScreenName
    destHtmlPath = destData.destHtmlPath
    local destWidth = destData.destTextureWidth or 256
    local destHeight = destData.destTextureHeight or 16
    local destText = destData.destText or ""
    local destRoute = destData.destRoute or ""
    local destExtra1 = destData.destExtra1 or ""
    local destExtra2 = destData.destExtra2 or ""
    local destExtra1r = destData.destExtra1r or ""
    local destExtra2r = destData.destExtra2r or ""
    local destInt = destData.destInt or 3
    local destImg = destData.destImg or 0
    local destExtra1Img = destData.destExtra1Img or 0
    local destExtra2Img = destData.destExtra2Img or 0
    local routeImg = destData.routeImg or 0
    local textColour = destData.textColour or textColourdef
    textColour = split(textColour, ",", textColourdef)
    local bgColour = destData.bgColour or bgColourdef
    bgColour = split(bgColour, ",", bgColourdef)
    local invert = destData.invert or invertdef
    invert = split(invert, ",", invertdef)
    local useroutecolour = destData.useroutecolour or 1
	colourSign = destData.colourSign or false
	signcolour = destData.signcolour or ""
	local transition = destData.transition or 0
	local transitionspd = destData.transitionspd or 1

    if destHtmlPath and destScreenName then
      htmlTexture.create(destScreenName, destHtmlPath, destWidth, destHeight, 20, 'automatic')
		eventDataNew = {direction = destText, routeID = destRoute, routeId = destRoute, displaytext = destText, displaytextrear = destRoute, extratext1 = destExtra1, extratext2 = destExtra2, interval = destInt, extratext1r = destExtra1r, extratext2r = destExtra2r, displayimg = destImg, destimg = destImg, extratext1img = destExtra1Img, extratext2img = destExtra2Img, routeimg = routeImg, coloursign = colourSign, textcolour = textColour, displaytextcolour = textColour[2], displaytextcolourrear = textColour[1], routetextcolour = textColour[1], bgcolour = bgColour, displaybgcolour = bgColour[2], displaybgcolourrear = bgColour[1], routebgcolour = bgColour[1], invert = invert, displayinvert = invert[2], displayinvertrear = invert[1], routeinvert = invert[1], useroutecolour = useroutecolour, signcolour = signcolour, transition = transition, transitionspd = transitionspd}
		eventDataOld = eventDataNew
      htmlTexture.call(destScreenName, "update", eventDataOld )
    else
      log("E", logTag, "Got no html path or material name for destination, no HTML texture created!!!...")
    end
  end
end

local function onGameplayEvent(eventName, eventData)
  if eventName == "bus_onRouteChange" then
    if destScreenName then
		eventDataNew = eventData
		if not eventData.direction then eventDataNew.direction = eventDataOld.direction end
		if not eventData.routeColor then eventDataNew.routeColor = eventDataOld.routeColor end
		eventDataOld = eventDataNew
		eventDataNew.coloursign = colourSign
		eventDataOld.coloursign = colourSign
		eventDataNew.signcolour = signcolour
		eventDataOld.signcolour = signcolour
		destScreenCount = 1
		destScreenCountrear = 1
		  if eventDataOld.routeID == "" or eventDataOld.routeId == "" then
			destScreenCountrear = 2
		  end
		destScreenTimer = tonumber(eventDataOld.interval)
		displaytextarray = {}
		displaytextcolourarray = {}
		displaybgcolourarray = {}
		displayimgarray = {}
		displaytextreararray = {}
		displaytextcolourreararray = {}
		displaybgcolourreararray = {}
		displayinvertarray = {}
		displayinvertreararray = {}
		  table.insert(displaytextarray, eventDataOld.direction)
		  table.insert(displaytextcolourarray, split(eventDataOld.textcolour,",",textColourdef)[2])
		  table.insert(displaybgcolourarray, split(eventDataOld.bgcolour,",",bgColourdef)[2])
		  table.insert(displayinvertarray, split(eventDataOld.invert,",",invertdef)[2])
		  if eventDataOld.extratext1 ~= "" then
			table.insert(displaytextarray, eventDataOld.extratext1)
			table.insert(displaytextcolourarray, split(eventDataOld.textcolour,",",textColourdef)[3])
			table.insert(displaybgcolourarray, split(eventDataOld.bgcolour,",",bgColourdef)[3])
			table.insert(displayinvertarray, split(eventDataOld.invert,",",invertdef)[3])
		  end
		  if eventDataOld.extratext2 ~= "" then
			table.insert(displaytextarray, eventDataOld.extratext2)
			table.insert(displaytextcolourarray, split(eventDataOld.textcolour,",",textColourdef)[4])
			table.insert(displaybgcolourarray, split(eventDataOld.bgcolour,",",bgColourdef)[4])
			table.insert(displayinvertarray, split(eventDataOld.invert,",",invertdef)[4])
		  end
		  table.insert(displayimgarray, eventDataOld.destimg)
		  if eventDataOld.extratext1 ~= "" then table.insert(displayimgarray, eventDataOld.extratext1img) end
		  if eventDataOld.extratext2 ~= "" then table.insert(displayimgarray, eventDataOld.extratext2img) end
			table.insert(displaytextreararray, eventDataOld.routeID or eventDataOld.routeId)
			table.insert(displaytextcolourreararray, split(eventDataOld.textcolour,",",textColourdef)[1])
			table.insert(displaybgcolourreararray, split(eventDataOld.bgcolour,",",bgColourdef)[1])
			table.insert(displayinvertreararray, split(eventDataOld.invert,",",invertdef)[1])
		  if eventDataOld.extratext1r ~= "" then
			table.insert(displaytextreararray, eventDataOld.extratext1r)
			table.insert(displaytextcolourreararray, split(eventDataOld.textcolour,",",textColourdef)[5])
			table.insert(displaybgcolourreararray, split(eventDataOld.bgcolour,",",bgColourdef)[5])
			table.insert(displayinvertreararray, split(eventDataOld.invert,",",invertdef)[5])
		  end
		  if eventDataOld.extratext2r ~= "" then
			table.insert(displaytextreararray, eventDataOld.extratext2r)
			table.insert(displaytextcolourreararray, split(eventDataOld.textcolour,",",textColourdef)[6])
			table.insert(displaybgcolourreararray, split(eventDataOld.bgcolour,",",bgColourdef)[6])
			table.insert(displayinvertreararray, split(eventDataOld.invert,",",invertdef)[6])
		  end
		  if #(displaytextreararray) == 0 then table.insert(displaytextreararray, "") end
		  eventDataNew.displaytext = displaytextarray[1]
		  eventDataNew.displaytextcolour = displaytextcolourarray[1]
		  eventDataNew.displaybgcolour = displaybgcolourarray[1]
		  eventDataNew.displayimg = displayimgarray[1]
		  eventDataNew.displaytextrear = displaytextreararray[1]
		  eventDataNew.displaytextcolourrear = displaytextcolourreararray[1]
		  eventDataNew.displaybgcolourrear = displaybgcolourreararray[1]
		  eventDataNew.routetextcolour = displaytextcolourreararray[1]
		  eventDataNew.routebgcolour = displaybgcolourreararray[1]
		  eventDataNew.displayinvert = displayinvertarray[1]
		  eventDataNew.displayinvertrear = displayinvertreararray[1]
		  eventDataNew.routeinvert = displayinvertreararray[1]
		  eventDataNew.destScreenCountrear = destScreenCountrear
		htmlTexture.call(destScreenName, "update", eventDataNew)
    end
    guihooks.trigger("BusDisplayUpdate", eventData)
    dump(eventData)
  end
end

M.init = init
M.updateGFX = updateGFX
M.onGameplayEvent = onGameplayEvent

return M
